from carehttp import Carehttp


def check_exist_by_param(cls, check_param):
    """Check if already scraped"""
    api_url = f"{cls.wp_root_url}/exist_api.php?source={check_param}"

    r = Carehttp().get(api_url, timeout=30, allow_redirects=False, verify=cls.verify)
    if r.status_code == 200:
        if r.text == "aExist":
            return True
        elif r.text == "aNotExist":
            return False
        else:
            raise Exception("Invalid response")
    else:
        raise Exception(f"Status is {r.status_code}")


import re
import arrow
from loguru import logger
from mewtools.mydatetime import core


# def get_datetime(text, custom_timezones=None, day_first=False):
#     """
#     Automatically convert/extract datetime from a text,
#     If you not sure about the format you can use this function,
#     Otherwise please use directly_datetime()/extract_datetime(), that's faster
#     ** Beware of that extract can't get tzinfo **
#     """
#     the_datetime = directly_datetime(text, custom_timezones, day_first=day_first)
#     if the_datetime:
#         # 直接转换时间
#         return the_datetime
#     else:
#         # 从文本抽取时间
#         return core.extract(text, day_first=day_first, method='datetime')


def directly_datetime(text, custom_timezones=None, day_first=False):
    """
    Convert a pure time string to datetime
    :param text: a pure datetime text
    :param custom_timezones: 自定义时区偏移字典，用来覆盖 timezone.py 里面的数据，列如 timezones_dic = {"CST": 8 * 3600}
    :param day_first:
    :return: datetime with tzinfo
    """
    return core.str2datetime(text, custom_timezones, day_first)


def extract_datetime(text, day_first=False):
    """
    Extract datetime from a text
    Beware of that extract can't get tzinfo
    :param text: a content include datetime text
    :param day_first:
    :return: datetime without tzinfo
    """
    return core.extract(text, day_first=day_first, method="datetime")


def extract_date(text, date_only=False, day_first=False):
    """
    Extract only date from a text
    :param text: a content include date/datetime text
    :param date_only: only return date string, or else with zero time
    :param day_first:
    :return: datetime without time and tzinfo
    """
    datetime_tmp = core.extract(text, day_first=day_first, method="date")

    if date_only:
        date = datetime_tmp.date()
        return date
    else:
        return datetime_tmp.replace(hour=0, minute=0, second=0)


def convert_timezone(old_datetime, city_name_or_timezone="local"):
    """
    Convert a datetime to another timezone
    :param old_datetime:
    :param city_name_or_timezone: 'Asia/Shanghai' or '+04:00' or 'local'
    :return:
    """
    try:
        if old_datetime.year < 1970:
            # 1970年之前的时间，arrow库不支持
            logger.warning(
                f"mewtools can not convert timezone for {old_datetime}, because it is before 1970"
            )
            return old_datetime

        new_datetime = arrow.get(old_datetime).to(city_name_or_timezone)
        return new_datetime.datetime
    except:
        raise Exception(f"datetime {old_datetime} might miss tzinfo")


def to_datetime_str(datetime_obj, format_rule="YYYY-MM-DD HH:mm:ss"):
    """
    Convert a datetime to string
    :param datetime_obj:
    :param format_rule:
    :return:
    """
    return arrow.get(datetime_obj).format(format_rule)



# coding=utf-8
from mylibtool.wordpress import custom
from mylibtool.wordpress import files
from mylibtool.wordpress import posts
from mylibtool.wordpress import category


class WpApi:
    def __init__(self, wp_url, wp_user, api_key, wp_user_id=1, default_cat=1, verify=True):
        if wp_url.endswith('/'):
            wp_url = wp_url[:-1]

        self.wp_root_url = wp_url
        self.wp_user = wp_user
        self.wp_user_id = wp_user_id
        self.api_url = f"{wp_url}/wp-json/wp/v2"
        self.api_key = api_key
        self.default_cat_id = default_cat
        self.verify = verify

    def post_file(self, target_file, title):
        return files.post_file(cls=self, target_file=target_file, title=title)

    def post_normal_article(self, article, title, cat_id=1, feature_pic=None, post_type='posts'):
        """
        Rest api post article
        :return:
        """
        return posts.post_normal_article(cls=self, article=article, title=title, cat_id=cat_id, feature_pic=feature_pic, post_type=post_type)

    def submit(self, payload, status='publish', post_type='posts'):
        """
        Submit news to WP
        :param payload: Api data, Can be dict or Addict
        :param status: Post status, publish, pending, draft
        :param post_type: Default is posts, you can post videos, pages, custom post type
        :return: Json/False
        """
        return posts.submit(cls=self, payload=payload, status=status, post_type=post_type)

    def cat_create(self, cat_name, cat_parent_id=0):
        """
        Create a category
        :return: category id
        """
        return category.create(cls=self, cat_name=cat_name, cat_parent_id=cat_parent_id)

    def is_post_exist(self, check_param):
        """Check if already scraped"""
        return custom.check_exist_by_param(self, check_param)



import re
import time
import maya

from datetime import datetime
from datetime import timedelta
from dateutil import parser
from date_extractor import extract_dates
from mylibtool.mydatetime import timezone

timestamp_reg = re.compile(r"^(\d{10,})$|"  # 10位时间戳
                           r"^(\d{13,})$|"  # 13位时间戳
                           r"^(\d{10,}\.\d{1,7})$")  # 这种形式只匹配最近的时间戳，由1开始，列如 1654280040

direct_pattern = re.compile(r"^(\d{10,})$|"  # 10位时间戳
                            r"^(\d{13,})$|"  # 13位时间戳
                            r"^(1\d{9,}\.\d{1,3})$|"  # 这种形式只匹配最近的时间戳，由1开始，列如 1654280040
                            r"^(\d{4}-\d{2}-\d+)$|"
                            r"^(\d{4}-\d{2}-\d+ \d{2})$|"
                            r"^(\d{4}-\d{2}-\d+ \d{2}:\d{2})$|"
                            r"^(\d{4}-\d{2}-\d+ \d{2}:\d{2}:\d{2})$")


def check_format(text):
    text_termed = replace_comma(text)

    if direct_pattern.search(text_termed):
        print('match')


def replace_comma(text):
    """Comma filter"""
    commas = {
        "年": "-",
        "月": "-",
        "日": " ",
        "时": ":",
        "分": ":",
        "秒": " ",
        "(": " ",
        ")": " ",
    }
    rs = map(lambda x: commas[x] if x in commas.keys() else x, list(text))
    return "".join(list(rs))


def replace_tzinfo(text):
    # 替换成偏移量
    pass


def is_pure_datetime(text):
    rule = re.compile(r"^[\d.,: at|Sun|Mon|Tue|Wed|Thu|Fri|Sat|January|-]+$")
    if rule.search(text):
        return True


def get_datetime(text):
    """Convert or extract datetime"""
    the_datetime = directly_datetime(text)
    if the_datetime:
        # 直接转换时间
        return the_datetime
    else:
        # 从文本抽取时间
        return extract(text)


def extract(text):
    date_tmp = extract_dates(text)
    if date_tmp:
        return date_tmp[0]


def directly_datetime(text):
    """
    用两种方法来转换直接的时间字符串
    :param text:
    :return:
    """
    if len(text) > 50:
        # 太长的不可能时直接时间
        return False

    # 判断是否时间戳
    if timestamp_reg.search(text):
        if re.search(r'^\d+$', text):
            # 不带小数点的13位时间戳转换位10位
            text = text[:10]

        ts = int(float(text))
        return maya.MayaDT(ts).datetime()

    text = replace_comma(text)  # 替换中文等干扰

    timezones_dic = timezone.timezones_dic
    the_datetime = None
    maya_datetime = None
    dateutil_datetime = None

    try:
        maya_tmp = maya.parse(text)
        if maya_tmp.epoch < 0:
            # Convert if date earlier than 1970
            maya_datetime = datetime(1970, 1, 1) - timedelta(seconds=abs(maya_tmp.epoch))
        else:
            maya_datetime = maya_tmp.datetime()
    except:
        pass

    try:
        dateutil_datetime = parser.parse(text, tzinfos=timezones_dic)
    except:
        pass

    if dateutil_datetime and maya_datetime and dateutil_datetime.tzinfo:
        # 将 dateutil拿到的时区写到 maya拿到的时间
        maya_datetime = maya_datetime.replace(tzinfo=dateutil_datetime.tzinfo)

    return maya_datetime


if __name__ == '__main__':
    time_list = [
        "1994-02-21T12:00:00+05:30",
        '1507150827',
        '1607150827.111',
        '1607150827111',
        '1970年01月01日3时00分00秒',
        '1970年01月01日',
        '1/2/2000',
        '01/02/2000',
        '31/01/2000T',
        '31/1/2000',
        '01/30/2000',
        '29-02-2000',
        '29/Mar/2020',
        '29/Mar/22',
        '8 Aug 2020',
        'Dec-01-2000',
        '2020.1.29',
        '2020.01.29',
        '2020.1.8 2:30',
        '2020.1.8 02:30:01',
        '2020.1.8 02:30:01.000',
        '2020.1.8 2am',
        '2020.1.8 2pm',
        '10 Jan. 1896',
        'Jan.26 1896',
        '2016-08-9T10:01:54.123Z ',
        '20160809',
        '1896-10-30',
        'Jan 2009',
        '1992/02/12 12:23:22+0800',
        '2016-08-06 8:00PM CST',
        '2016-08-06 08:00PM Z',
        'Jinstin said at 2020-10-30',
    ]
    for item in time_list:
        # t = get_datetime(item)
        t = directly_datetime(item)
        print(t)



import json

from mylibtool.wordpress import files
from loguru import logger
from carehttp import Carehttp


def post_normal_article(cls, article, title, cat_id=1, feature_pic=None, post_type='posts'):
    """
    Rest api post article
    :param article: an addict param
    :return: (post id, post link)
    """
    if feature_pic:
        # upload if need feature pic
        wp_file = files.post_file(
            cls=cls,
            target_file=feature_pic,
            title=title
        )

        if not wp_file:
            logger.error(f'upload pic: {title} failed')
            return False

        file_id, file_url = wp_file

        article.featured_media = file_id

    try:
        # submit a post
        article.categories = cat_id
        json_data = submit(cls=cls, payload=article, post_type=post_type)

        if json_data and isinstance(json_data, dict):
            logger.success(f'Submit {title} succeed')
            return json_data['id']

    except Exception as e:
        logger.error(f'Submit {title} failed: {e}')
        return


def submit(cls, payload, status='publish', post_type='posts'):
    """
    Submit posts to WP
    """
    api_url = f'{cls.api_url}/{post_type}'

    # Add basic params to payload
    payload['status'] = status
    payload['author'] = cls.wp_user_id

    logger.debug(f"Submit Article: {payload['title']}")

    r = Carehttp(payload['title'], tries=8).post(url=api_url,
                                                 data=json.dumps(payload),
                                                 headers={'content-type': "Application/json"},
                                                 auth=(cls.wp_user, cls.api_key),
                                                 verify=cls.verify
                                                 )

    if r.status_code == 201:
        logger.success(f'Submit success: {r.status_code}')
        return r.json()
    else:
        logger.error(f'Submit failed: {r.status_code} - {r.text}')
        return False



import time
import filetype

from addict import Dict
from loguru import logger
from carehttp import Carehttp
from requests_toolbelt.multipart.encoder import MultipartEncoder


def post_file(cls, target_file, title):
    """
    Download and upload picture to WP
    :param cls: wordpress object
    :param target_file: file url or local path
    :param title:
    :return: (file_id, file_wp_url)
    """
    file_content = None

    # if remote pic, download it, if local path, read it
    if target_file.startswith('http'):
        try:
            # download file
            r_download = fetch_pic(url=target_file)
            if r_download.status_code != 200:
                logger.error(f'Download {target_file} failed: {r_download.status_code}')
                return

            file_content = r_download.content
        except Exception as e:
            logger.error(f'Download {target_file} failed: {e}')
            return
    else:
        try:
            # open local file
            with open(target_file, 'rb') as f:
                file_content = f.read()
        except Exception as e:
            logger.error(f"Open file {target_file} failed: {e}")
            return

    try:
        # get file's mine and suffix
        kind = filetype.guess(file_content)
        if kind is None:
            raise

    except:
        logger.error(f"Can not guess file type")
        return

    try:
        new_file = {
            'title': title,
            'file_name': f'autoPost_{str(round(time.time() * 1000))}.{kind.extension}',  # random a new file name
            'mine': kind.mime,
            'file_content': file_content,
        }
        r_upload = upload2wp(cls=cls, file_obj=new_file)
    except Exception as e:
        logger.error(f'Upload {target_file} failed: {e}')
        return

    if r_upload.status_code != 201:
        logger.error(f'Upload {target_file} failed:  {r_upload.status_code}')
        return

    json_data = r_upload.json()

    return json_data['id'], json_data['source_url']


def fetch_pic(url):
    header = {
        'Connection': 'close',
        'accept': 'image/avif,image/webp,image/apng,image/svg+xml,image/*,*/*;q=0.8',
        'accept-encoding': 'gzip,deflate,br',
        'user-agent': 'Mozilla/5.0(WindowsNT10.0;Win64;x64)AppleWebKit/537.36(KHTML,likeGecko)Chrome/95.0.4638.69Safari/537.36',
    }

    logger.debug(f"Download: {url}")
    r = Carehttp(tries=6).get(url, headers=header, timeout=60, allow_redirects=False)
    return r


def upload2wp(cls, file_obj):
    """
    Upload picture to WP
    :param cls: class object
    :param file_obj: dict - {title, file_name, mine, file_content}
    :return: Json/None
    """
    api_url = f'{cls.api_url}/media'

    multipart_data = MultipartEncoder(
        fields={
            # a file upload field
            'file': (file_obj['file_name'], file_obj['file_content'], file_obj['mine']),
            # plain text fields
            'caption': file_obj['title'],
        }
    )

    r = Carehttp(file_obj['file_name']).post(url=api_url,
                                             data=multipart_data,
                                             headers={'Content-Type': multipart_data.content_type},
                                             auth=(cls.wp_user, cls.api_key),
                                             verify=cls.verify
                                             )

    return r



import json
from loguru import logger
from carehttp import Carehttp


def create(cls, cat_name, cat_parent_id=0):
    """Create and return the category ID, if the category exists, return the category ID"""
    api_url = f'{cls.api_url}/categories'

    payload = {
        'name': cat_name,
        'parent': cat_parent_id,
    }

    headers = {'content-type': "Application/json"}

    r = Carehttp(f'Create {cat_name}').post(url=api_url,
                                            data=json.dumps(payload),
                                            headers=headers,
                                            auth=(cls.wp_user, cls.api_key),
                                            verify=False,
                                            )

    if r.status_code == 201:
        logger.debug(f'Create {cat_name} success: {r.status_code}')
        return r.json()['id']
    else:
        if r.status_code == 400 and 'term_exists' in r.text:
            logger.warning(f'{cat_name} category exists')
            return r.json()['data']['term_id']

        logger.error(f'Create {cat_name} failed: {r.status_code}')
        return False



"""提取 datetime 前对文本的调整"""
import re
import datetime
from loguru import logger
from mylibtool.mydatetime import month_pair


def _filter_middle_part(text):
    """过滤日期与时间部分的干扰"""
    if re.search(r'\d{1,2} at \d{1,2}', text):
        text = text.replace(' at ', ' ')

    return text


def format2year_month_day(text, day_first):
    """格式化非标准的日期为 yy-mm-dd 格式"""
    text = f' {text} '  # 前后加空格，防止首尾为^或$

    text = _replace_named_month(text)  # 把英文月份转换成数字月份

    '''↓ mm/dd/yy | dd/mm/yy 格式处理开始 ↓'''

    def _to_ymd(matched):
        non_ymd_list = list(matched.groups())
        if day_first:
            # 默认遵循月份在前，所以 dayfirst 的情况需要交换两个的位置
            non_ymd_list[0], non_ymd_list[2] = non_ymd_list[2], non_ymd_list[0]

        # 判断 date 有效性，并尝试更正，无法更正就让 extract 无效
        try:
            non_ymd_list = _to_ymd_check_valid(non_ymd_list)
        except Exception as e:
            logger.warning(e)
            return 'Date Invalid'

        ymd_str = f'{non_ymd_list[3]}-{non_ymd_list[0]}-{non_ymd_list[2]} {non_ymd_list[-1]}'
        return ymd_str

    text = re.sub(r'(\d{1,2})([ /.·-]+)(\d{1,2})\2(\d{4})[^\d](\d{1,2}:\d{2})', _to_ymd, text)
    '''↑ mm/dd/yy | dd/mm/yy 格式处理结束 ↑'''

    '''↓ yy-mm-dd 格式处理开始 ↓'''
    text = re.sub(r'\D([12]\d{3})([/.·-]+)(\d)\2(\d{2})\D', r' \1-0\3-\4 ', text)  # 修改 单位数月份为双位数
    text = re.sub(r'\D([12]\d{3})([/.·-]+)(\d{2})\2(\d)\D', r' \1-\3-0\4 ', text)  # 修改 单位数日期为双位数

    text = re.sub(r'([12]\d{3})([ /.·]+)(\d{2})\2(\d{2})\D', r'\1-\3-\4 ', text)  # 修改其他日期分隔符为-

    text = _filter_middle_part(text)  # 过滤日期和时间中间的干扰

    time_reg = re.compile(r'(\d{4}-\d{2}-\d{2}) {1,2}([0-2]?\d(:[0-6]\d){1,2})( ?am| ?pm| ?a.m.| ?p.m.|[^:\d])', re.I)  # 第一组为日期，第二组为时间，第三组为 am 或 pm
    text = re.sub(time_reg, _final_time_callback, text)  # 修改 单位数小时为双位，补全秒

    text = re.sub(r'(\D)(\d{4}-\d{2}-\d{2} [0-2]\d:[0-6]\d)', r';\2', text)  # 日期前面有数字会影响提取，此处用分隔符隔开，如 0.105 2021-04-01 to 0.105;2021-04-01
    '''↑ yy-mm-dd 格式处理结束 ↑'''

    return text


def _replace_named_month(text):
    """英文月份转换为数字月份，并且转化为 （年-月-日） 形式"""
    reg1 = re.compile(r'(?<=[^a-zA-Z])(January|February|March|April|May|June|July|August|September|October|November|December)(?=[/.,\d -]+)', re.I)  # 匹配月份全称
    step1_text = re.sub(reg1, _month_callback, text)  # 月份转换后用 '龘' 作为识别符

    reg1 = re.compile(r'(?<=[^a-zA-Z])(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\.?(?=[/.,\d -]+)', re.I)  # 匹配月份简写，可以匹配 Jan. 这样的简写。 https://regexr.com/6n41e
    step1_text = re.sub(reg1, _month_callback, step1_text)  # 月份转换后用 '龘' 作为识别符

    '''↓用非逗号和逗号分割月 日, 年的情况↓'''
    step1_text = re.sub(r'(\d{1,2}龘)[ .](\d{1,2}), ?(\d{4})', r'\1/\2/\3', step1_text)
    '''↑用非逗号和逗号分割月 日, 年的情况↑'''

    # 月份在中间的情况
    reg2 = re.compile(r'(\d{4}|\d{1,2})([ /.·-]+)(\d{2}龘)\2(\d{4}|\d{1,2})[\D]')  # 英文名月份很可能由空格分割
    step2_text = re.sub(reg2, _sort_ymd_by_mark_callback, step1_text)

    # 月份在前的情况
    reg3 = re.compile(r'(\d{2}龘)([ /.·-]+)(\d{4}|\d{1,2})\2(\d{4}|\d{1,2})[^:\d]')  # 英文名月份很可能由空格分割
    step3_text = re.sub(reg3, _sort_ymd_by_mark_callback, step2_text)

    return step3_text


def _sort_ymd_by_mark_callback(matched):
    """排序 yy-mm-dd，月份使用 龘 修饰的情况"""
    ymd_list = matched.groups()

    year = ''
    month = ''
    day = ''

    # 获取年月日
    for m in ymd_list:
        if len(m) == 4 and m.isdigit():
            year = m
        elif '龘' in m:
            month = m.replace('龘', '')
        elif len(m) <= 2 and m.isdigit():
            day = m

    return f'{year}-{month}-{day} '


def _month_callback(matched):
    """正则取英文月份，用（双位数字+龘）来替换成对应月份"""
    symbol = matched.group(1)
    if not symbol:
        return symbol

    if not symbol[0].isupper():
        # 首字母为小写，可能不是日期
        return symbol

    for m in month_pair.month_map:
        if m[0] == symbol.title() or m[1] == symbol.title():
            month_instead = m[2] + "龘"
            return month_instead


def _final_time_callback(matched):
    """对时间进行最终的调整，修改 单位数小时为双位，补全秒"""
    t_list = matched.groups()

    t_nums = t_list[1].split(':')  # 分割时间为数字

    def __include(str_list, *args):
        for k in args:
            for t in str_list:
                if k == t.lower():
                    return True

    if __include(t_list, 'am', ' am', ' a.m.', 'a.m.'):
        # 补全单位数的小时（上午）
        if len(t_nums[0]) == 1:
            t_nums[0] = f'0{t_nums[0]}'
    elif __include(t_list, 'pm', ' pm', ' p.m.', 'p.m.'):
        # 补全单位数的小时（下午）
        if int(t_nums[0]) < 12:
            pm_hour = str(int(t_nums[0]) + 12)
            t_nums[0] = pm_hour
    elif len(t_nums[0]) == 1:
        # 没有 am,pm 标识，但是时间为1位数
        t_nums[0] = f'0{t_nums[0]}'

    # 补全秒数
    if len(t_nums) == 2:
        t_nums.append('00')

    return f'{t_list[0]} {":".join(t_nums)}'


def _get_days(year, month):
    """获取特定月份的天数"""
    if month in (1, 3, 5, 7, 8, 10, 12):
        return 31
    elif month in (4, 6, 9, 11):
        return 30
    elif month == 2:
        if (year % 4 == 0 and year % 100 != 0) or (year % 400 == 0):
            return 29
        else:
            return 28
    else:
        raise Exception("Year Invalid")


def _to_ymd_check_valid(non_ymd_list):
    """
    判断 date 有效性，尝试更正
    返回更正后的 list, 若更正失败则触发异常
    """
    year = int(non_ymd_list[3])  # 年
    month = int(non_ymd_list[0])  # 月
    day = int(non_ymd_list[2])  # 日
    if (month > 12) and (day < 13):
        # 月大于12，但是日小于12，那么月和日应该是反了
        non_ymd_list[0], non_ymd_list[2] = non_ymd_list[2], non_ymd_list[0]
        # 月、日重新取值
        month = int(non_ymd_list[0])
        day = int(non_ymd_list[2])

    # 日期超过该月合理天数则无效
    if day > _get_days(int(year), int(month)):
        raise Exception("Day Invalid")

    # 月份有效判断
    if month > 12:
        raise Exception("Month Invalid")

    # 年有效判断
    if year > 2050:
        raise Exception("Year over 2050")

    return non_ymd_list


def timez_str_convert(text):
    """text 中时间字符串转换成偏移格式"""
    text = re.sub(r'(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2}) GMT([+-])([:\d]+)', _gmt2offset, text)
    return text


def _datetime_math(time_list, offset_symbol, offset_h, offset_m):
    """datetime 加减"""
    the_datetime = datetime.datetime.now().replace(
        year=time_list['yy'],
        month=time_list['mm'],
        day=time_list['dd'],
        hour=time_list['h'],
        minute=time_list['m'],
        second=time_list['s'],
        microsecond=0
    )  # 用 string 转成 datetime

    if offset_symbol == '-':
        modified_datetime = the_datetime + datetime.timedelta(minutes=offset_m)
        modified_datetime = modified_datetime + datetime.timedelta(hours=offset_h)
    else:
        modified_datetime = the_datetime - datetime.timedelta(minutes=offset_m)
        modified_datetime = modified_datetime - datetime.timedelta(hours=offset_h)

    return modified_datetime


def _gmt2offset(matched):
    t_list = matched.groups()

    # 年、月、日、时、分
    time_list = {
        'yy': int(t_list[0]),
        'mm': int(t_list[1]),
        'dd': int(t_list[2]),
        'h': int(t_list[3]),
        'm': int(t_list[4]),
        's': int(t_list[5]),
    }
    offset_symbol = t_list[-2]  # 正负符号

    if len(t_list[-1]) == 1:
        # 偏移只有小时位，偏移小时和偏移分钟
        offset_h = int(t_list[-1])
        offset_m = 0
    else:
        # 偏移有小时位和分钟位
        if ':' in t_list[-1]:
            hm = t_list[-1].split(':')
            offset_h = int(hm[0])
            offset_m = int(hm[1])
        else:
            offset_h = int(t_list[-1][0:1])
            offset_m = int(t_list[-1][2:3])

    modified_datetime = _datetime_math(time_list, offset_symbol, offset_h, offset_m)

    modified_datetime_str = f'{modified_datetime.strftime("%Y-%m-%d %H:%M:%S")}{offset_symbol}{str(offset_h).zfill(2)}{str(offset_m).zfill(2)}'

    return modified_datetime_str




import re
import maya
import warnings

from datetime import datetime
from datetime import timedelta
from dateutil import parser
from date_extractor import extract_dates
from mylibtool.mydatetime import timezone
from mylibtool.mydatetime import month_pair
from mylibtool.mydatetime import extract_prep


timestamp_reg = re.compile(r"^(\d{10,})$|"  # 10位时间戳
                           r"^(\d{13,})$|"  # 13位时间戳
                           r"^(\d{10,}\.\d{1,7})$")  # 这种形式只匹配最近的时间戳，由1开始，列如 1654280040


def replace_comma(text):
    """清除常见干扰"""
    text = re.sub(r'日\D', r'日', text)  # 去掉 日后面的非数字干扰，列如：2022年7月12日T20:50:00

    commas = {
        "年": "-",
        "月": "-",
        "日": " ",
        "时": ":",
        "分": ":",
        "秒": " ",
        "(": " ",
        ")": " ",
    }
    rs = map(lambda x: commas[x] if x in commas.keys() else x, list(text))
    return "".join(list(rs))


def extract(text, day_first, method='date'):
    """
    从文本提取 datetime, 只有 date_extractor 库有提取功能
    extract_dates 函数提取日期默认是月份在前，列如：04-10-2013 提取为 2013-04-10
    :param text:
    :param method: 提取日期+时间还是只提取日期
    :param day_first: 字符串里，日期在前，默认则是月份在前
    :return:
    """
    if method == 'datetime':
        # 提取日期+时间
        text = replace_comma(text)
        text = _term_day(text)

        formatted_text = extract_prep.format2year_month_day(text, day_first)

        # 经过以上多重过滤后，先尝试使用 directly_datetime 来转换
        # try_directly_datetime = str2datetime(text, None, day_first)
        # if try_directly_datetime:
        #     print(f'-- {try_directly_datetime}')
        # else:
        #     formatted_text = extract_prep.timez_str_convert(formatted_text)

        date_tmp = extract_dates(formatted_text)
        if date_tmp:
            return date_tmp[0]

    else:
        # 只提取日期
        text = replace_comma(text)
        text = _term_day(text)

        if day_first:
            # day 在前需要调换顺序
            text = _interchange_month_day(text)

        date_tmp = extract_dates(text)
        if date_tmp:
            return date_tmp[0]


def _interchange_month_day(text):
    """
    因为 date_extractor 只能解析月份在前的格式
    所以日期在前的 text 必须先交换日和月的位置才能被 date_extractor 解析
    """
    interchanged_text = re.sub(r'(0?[1-9]|[12][0-9]|3[01])[/.](0?[1-9]|1[012])[/.](\d{4})', r'\2/\1/\3', text)
    return interchanged_text


def _term_day(text):
    """
    修剪日期，将 1st, 2nd, 3rd, 4th-9th 精简为双位数字
    """
    termed_text = re.sub(r'(\d)(st|nd|rd|th)', r'0\1', text)
    return termed_text


def str2datetime(text, custom_timezones, day_first):
    """
    用 dateutil 和 maya 库转换直接的时间字符串成 datetime
    :param custom_timezones: 自定义时区偏移字典，用来覆盖 timezone.py 里面的数据，列如 timezones_dic = {"CST": 8 * 3600}
    :param text:
    :return:
    """
    if len(text) > 50:
        # 太长的不可能是直接时间
        return False

    if custom_timezones and not isinstance(custom_timezones, dict):
        raise Exception('custom_timezones must be a dict')

    if timestamp_reg.search(text):
        # 直接转换时间戳为 datetime
        if re.search(r'^\d+$', text):
            # 不带小数点的13位时间戳转换位10位
            text = text[:10]

        ts = int(float(text))
        return maya.MayaDT(ts).datetime()

    # 以下处理非时间戳的字符串
    text = replace_comma(text)  # 替换中文等干扰

    timezones_dic = timezone.timezones_dic
    maya_datetime = None
    dateutil_datetime = None

    """使用 maya 库转换时间"""
    try:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")  # 忽略警告

            maya_tmp = maya.parse(text, day_first=day_first)
            if maya_tmp.epoch < 0:
                # maya 不能转换 1970以前的 datetime，所以这样处理
                maya_datetime = datetime(1970, 1, 1) - timedelta(seconds=abs(maya_tmp.epoch))
            else:
                maya_datetime = maya_tmp.datetime()
    except:
        pass

    """使用 dateutil 库转换时间"""
    try:
        if custom_timezones:
            # 用自定义的时区偏移替换通用的时区偏移
            timezones_dic.update(custom_timezones)

        dateutil_datetime = parser.parse(text, tzinfos=timezones_dic, dayfirst=day_first)
    except:
        pass

    """融合结果，这样会更准确"""
    if dateutil_datetime and maya_datetime and dateutil_datetime.tzinfo:

        # 将 dateutil 拿到的时区融合到 maya 的 datetime
        maya_datetime = maya_datetime.replace(tzinfo=dateutil_datetime.tzinfo)

        # maya 库会把(12:00:00+06:00) 格式的时间转化为 UTC 时间，这会相差 tzinfo 个时间，所以需要 ±tzinfo 调整回来
        matched = re.search(r'\D(\d{2}:\d{2}(\.\d{3,6})?([+-])\d{2}:?\d{2})($|\D)', text)
        if matched:
            # 获取相对于 UTC 的时间差
            offset_tmp = str(maya_datetime.tzinfo)
            try:
                offset_seconds = int(offset_tmp.replace(')', '').split(',')[1])
                maya_datetime = maya_datetime + timedelta(seconds=offset_seconds)
            except:
                pass

    return maya_datetime



